@@ -188,7 +188,6 @@ static void per_loop_activate(PgPool *pool)
 	statlist_for_each_safe(item, &pool->waiting_client_list, tmp) {
 		client = container_of(item, PgSocket, head);
 		if (!statlist_empty(&pool->idle_server_list)) {
-
 			/* db not fully initialized after reboot */
 			if (client->wait_for_welcome && !pool->welcome_msg_ready) {
 				launch_new_connection(pool, /* evict_if_needed= */ true);
@@ -351,7 +350,7 @@ void per_loop_maint(void)
 		} else {
 			active_count += statlist_count(&login_client_list);
 		}
-		/* fallthrough */
+	/* fallthrough */
 	case P_PAUSE:
 		if (!active_count)
 			admin_pause_done();
@@ -404,16 +403,17 @@ static void pool_client_maint(PgPool *pool)
 				disconnect_client(client, true, "query_wait_timeout");
 			}
 		}
+	}
+
+	/* apply cancel_wait_timeout for cancel connections */
+	if (cf_cancel_wait_timeout > 0) {
 		statlist_for_each_safe(item, &pool->waiting_cancel_req_list, tmp) {
 			client = container_of(item, PgSocket, head);
 			Assert(client->state == CL_WAITING_CANCEL);
 			age = now - client->request_time;
 
-			if (cf_query_timeout > 0 && age > cf_query_timeout) {
-				disconnect_client(client, false, "cancel request query_timeout");
-			} else if (cf_query_wait_timeout > 0 && age > cf_query_wait_timeout) {
-				disconnect_client(client, false, "cancel request query_wait_timeout");
-			}
+			if (age > cf_cancel_wait_timeout)
+				disconnect_client(client, false, "cancel_wait_timeout");
 		}
 	}
 
@@ -430,6 +430,26 @@ static void pool_client_maint(PgPool *pool)
 	}
 }
 
+/* maintaining clients in peer pool */
+static void peer_pool_client_maint(PgPool *pool)
+{
+	struct List *item, *tmp;
+	usec_t now = get_cached_time();
+	PgSocket *client;
+	usec_t age;
+
+	if (cf_cancel_wait_timeout > 0) {
+		statlist_for_each_safe(item, &pool->waiting_cancel_req_list, tmp) {
+			client = container_of(item, PgSocket, head);
+			Assert(client->state == CL_WAITING_CANCEL);
+			age = now - client->request_time;
+
+			if (age > cf_cancel_wait_timeout)
+				disconnect_client(client, false, "cancel_wait_timeout");
+		}
+	}
+}
+
 static void check_unused_servers(PgPool *pool, struct StatList *slist, bool idle_test)
 {
 	usec_t now = get_cached_time();
@@ -495,8 +515,7 @@ static void check_pool_size(PgPool *pool)
 	    cur < pool_pool_size(pool) &&
 	    cf_pause_mode == P_NONE &&
 	    cf_reboot == 0 &&
-	    pool_client_count(pool) > 0)
-	{
+	    (pool_client_count(pool) > 0 || pool->db->forced_user != NULL)) {
 		log_debug("launching new connection to satisfy min_pool_size");
 		launch_new_connection(pool, /* evict_if_needed= */ false);
 	}
@@ -549,8 +568,7 @@ static void pool_server_maint(PgPool *pool)
 				disconnect_server(server, true, "query timeout");
 			} else if (cf_idle_transaction_timeout > 0 &&
 				   server->idle_tx &&
-				   age_server > cf_idle_transaction_timeout)
-			{
+				   age_server > cf_idle_transaction_timeout) {
 				disconnect_server(server, true, "idle transaction timeout");
 			}
 		}
@@ -573,6 +591,37 @@ static void pool_server_maint(PgPool *pool)
 	check_pool_size(pool);
 }
 
+/* maintain servers in a peer pool */
+static void peer_pool_server_maint(PgPool *pool)
+{
+	struct List *item, *tmp;
+	usec_t now = get_cached_time();
+	PgSocket *server;
+
+	/*
+	 * find connections that got connected, but could not log in. For normal
+	 * pools we only compare against server_connect_timeout for these servers,
+	 * but since peer pools are only for sending cancellations we also compare
+	 * against cancel_wait_timeout here.
+	 */
+	if (cf_server_connect_timeout > 0 || cf_cancel_wait_timeout > 0) {
+		statlist_for_each_safe(item, &pool->new_server_list, tmp) {
+			usec_t age;
+
+			server = container_of(item, PgSocket, head);
+			Assert(server->state == SV_LOGIN);
+
+			age = now - server->connect_time;
+			if (cf_server_connect_timeout > 0 && age > cf_server_connect_timeout) {
+				disconnect_server(server, true, "connect timeout");
+			} else if (cf_cancel_wait_timeout > 0 && age > cf_cancel_wait_timeout) {
+				disconnect_server(server, true, "cancel_wait_timeout");
+			}
+		}
+	}
+}
+
+
 static void cleanup_client_logins(void)
 {
 	struct List *item, *tmp;
@@ -631,6 +680,25 @@ static void do_full_maint(evutil_socket_t sock, short flags, void *arg)
 	if (cf_pause_mode == P_SUSPEND)
 		return;
 
+	/*
+	 * Creating new pools to enable `min_pool_size` enforcement even if
+	 * there are no active clients.
+	 *
+	 * If clients never connect there won't be a pool to maintain the
+	 * min_pool_size on, which means we have to proactively create a pool,
+	 * so that it can be maintained.
+	 *
+	 * We are doing this for databases with forced users only, to reduce
+	 * the risk of creating connections in unexpected ways, where there are
+	 * many users.
+	   _	 */
+	statlist_for_each_safe(item, &database_list, tmp) {
+		db = container_of(item, PgDatabase, head);
+		if (database_min_pool_size(db) > 0 && db->forced_user != NULL) {
+			get_pool(db, db->forced_user);
+		}
+	}
+
 	statlist_for_each_safe(item, &pool_list, tmp) {
 		pool = container_of(item, PgPool, head);
 		if (pool->db->admin)
@@ -645,6 +713,12 @@ static void do_full_maint(evutil_socket_t sock, short flags, void *arg)
 		}
 	}
 
+	statlist_for_each_safe(item, &peer_pool_list, tmp) {
+		pool = container_of(item, PgPool, head);
+		peer_pool_server_maint(pool);
+		peer_pool_client_maint(pool);
+	}
+
 	/* find inactive autodbs */
 	statlist_for_each_safe(item, &database_list, tmp) {
 		db = container_of(item, PgDatabase, head);
@@ -661,9 +735,9 @@ static void do_full_maint(evutil_socket_t sock, short flags, void *arg)
 
 	cleanup_client_logins();
 
-	if (cf_shutdown == 1 && get_active_server_count() == 0) {
+	if (cf_shutdown == SHUTDOWN_WAIT_FOR_SERVERS && get_active_server_count() == 0) {
 		log_info("server connections dropped, exiting");
-		cf_shutdown = 2;
+		cf_shutdown = SHUTDOWN_IMMEDIATE;
 		event_base_loopbreak(pgb_event_base);
 		return;
 	}
@@ -702,9 +776,29 @@ void kill_pool(PgPool *pool)
 	list_del(&pool->map_head);
 	statlist_remove(&pool_list, &pool->head);
 	varcache_clean(&pool->orig_vars);
+	slab_free(var_list_cache, pool->orig_vars.var_list);
 	slab_free(pool_cache, pool);
 }
 
+void kill_peer_pool(PgPool *pool)
+{
+	const char *reason = "peer removed";
+
+	close_client_list(&pool->active_cancel_req_list, reason);
+	close_client_list(&pool->waiting_cancel_req_list, reason);
+	close_server_list(&pool->active_cancel_server_list, reason);
+	close_server_list(&pool->new_server_list, reason);
+
+	pktbuf_free(pool->welcome_msg);
+
+	list_del(&pool->map_head);
+	statlist_remove(&peer_pool_list, &pool->head);
+	varcache_clean(&pool->orig_vars);
+	slab_free(var_list_cache, pool->orig_vars.var_list);
+	slab_free(peer_pool_cache, pool);
+}
+
+
 void kill_database(PgDatabase *db)
 {
 	PgPool *pool;
@@ -729,10 +823,33 @@ void kill_database(PgDatabase *db)
 	} else {
 		statlist_remove(&database_list, &db->head);
 	}
+
+	if (db->auth_dbname)
+		free((void *)db->auth_dbname);
+
 	aatree_destroy(&db->user_tree);
 	slab_free(db_cache, db);
 }
 
+void kill_peer(PgDatabase *db)
+{
+	PgPool *pool;
+	struct List *item, *tmp;
+
+	log_warning("dropping peer %s as it does not exist anymore", db->name);
+
+	statlist_for_each_safe(item, &peer_pool_list, tmp) {
+		pool = container_of(item, PgPool, head);
+		if (pool->db == db)
+			kill_peer_pool(pool);
+	}
+
+	free(db->host);
+
+	statlist_remove(&peer_list, &db->head);
+	slab_free(peer_cache, db);
+}
+
 /* as [pgbouncer] section can be loaded after databases,
    there's need for review */
 void config_postprocess(void)
@@ -747,4 +864,12 @@ void config_postprocess(void)
 			continue;
 		}
 	}
+
+	statlist_for_each_safe(item, &peer_list, tmp) {
+		db = container_of(item, PgDatabase, head);
+		if (db->db_dead) {
+			kill_peer(db);
+			continue;
+		}
+	}
 }
